@@ -60,7 +60,7 @@
  * Constructs a mixer window (KMix main window)
  */
 KMixWindow::KMixWindow()
-	: TDEMainWindow(0, 0, 0, 0), m_showTicks( true ),
+	: DCOPObject("kmix"), TDEMainWindow(0, 0, 0, 0), m_showTicks( true ),
 	m_dockWidget( 0L )
 {
 	m_visibilityUpdateAllowed	= true;
@@ -120,11 +120,11 @@ KMixWindow::initActions()
 
 	m_globalAccel = new TDEGlobalAccel(this, "KMix");
 	m_globalAccel->insert( "Increase volume", i18n( "Increase Volume of Master Channel"), TQString(),
-			TDEShortcut(TQString("XF86AudioRaiseVolume")), TDEShortcut(TQString("XF86AudioRaiseVolume")), TQT_TQOBJECT(this), TQT_SLOT( increaseVolume() ) );
+			TDEShortcut(), TDEShortcut(), TQT_TQOBJECT(this), TQT_SLOT( slotIncreaseVolume() ) );
 	m_globalAccel->insert( "Decrease volume", i18n( "Decrease Volume of Master Channel"), TQString(),
-			TDEShortcut(TQString("XF86AudioLowerVolume")), TDEShortcut(TQString("XF86AudioLowerVolume")), TQT_TQOBJECT(this), TQT_SLOT( decreaseVolume() ) );
+			TDEShortcut(), TDEShortcut(), TQT_TQOBJECT(this), TQT_SLOT( slotDecreaseVolume() ) );
 	m_globalAccel->insert( "Toggle mute", i18n( "Toggle Mute of Master Channel"), TQString(),
-			TDEShortcut(TQString("XF86AudioMute")), TDEShortcut(TQString("XF86AudioMute")), TQT_TQOBJECT(this), TQT_SLOT( toggleMuted() ) );
+			TDEShortcut(), TDEShortcut(), TQT_TQOBJECT(this), TQT_SLOT( slotToggleMuted() ) );
 	m_globalAccel->readSettings();
 	m_globalAccel->updateConnections();
 
@@ -154,7 +154,7 @@ KMixWindow::initWidgets()
 
 	// Mixer widget line
 	mixerNameLayout = new TQHBox( centralWidget(), "mixerNameLayout" );
-        widgetsLayout->setStretchFactor( mixerNameLayout, 0 );
+	widgetsLayout->setStretchFactor( mixerNameLayout, 0 );
 	TQSizePolicy qsp( TQSizePolicy::Ignored, TQSizePolicy::Maximum);
 	mixerNameLayout->setSizePolicy(qsp);
 	mixerNameLayout->setSpacing(KDialog::spacingHint());
@@ -169,7 +169,7 @@ KMixWindow::initWidgets()
 	widgetsLayout->addWidget( mixerNameLayout );
 
 	m_wsMixers = new TQWidgetStack( centralWidget(), "MixerWidgetStack" );
-        widgetsLayout->setStretchFactor( m_wsMixers, 10 );
+	widgetsLayout->setStretchFactor( m_wsMixers, 10 );
 	widgetsLayout->addWidget( m_wsMixers );
 
 	if ( m_showMenubar )
@@ -286,10 +286,14 @@ KMixWindow::loadConfig()
    m_surroundView    = config->readBoolEntry("Experimental-ViewSurround", false );
    m_gridView    = config->readBoolEntry("Experimental-ViewGrid", false );
    const TQString& orientationString = config->readEntry("Orientation", "Horizontal");
-   TQString mixerMasterCard = config->readEntry( "MasterMixer", "" );
-   Mixer::setMasterCard(mixerMasterCard);
-   TQString masterDev = config->readEntry( "MasterMixerDevice", "" );
-   Mixer::setMasterCardDevice(masterDev);
+   bool useDefaultMaster = config->readBoolEntry( "UseDefaultMaster", true );
+   if (!useDefaultMaster)
+   {
+       TQString mixerMasterCard = config->readEntry( "MasterMixer", "" );
+       Mixer::setMasterCard(mixerMasterCard);
+       TQString masterDev = config->readEntry( "MasterMixerDevice", "" );
+       Mixer::setMasterCardDevice(masterDev);
+   }
 
    if ( valueStyleString == "Absolute" )
        m_valueStyle = MixDeviceWidget::NABSOLUTE;
@@ -595,41 +599,163 @@ KMixWindow::configureGlobalShortcuts()
         m_globalAccel->updateConnections();
 }
 
-void
-KMixWindow::toggleMuted()
+// KMixIface DCOP interface methods
+void KMixWindow::setVolume(int percentage)
 {
-   Mixer* mixerMaster = Mixer::masterCard();
-   if ( mixerMaster != 0 ) {
-      MixDevice* md = mixerMaster->masterDevice();
-      if ( md != 0 && md->hasMute() ) {
-         mixerMaster->toggleMute(md->num());
+   Mixer *mixerMaster = Mixer::masterCard();
+   if (mixerMaster)
+   {
+      mixerMaster->setMasterVolume(percentage);
+   }
+}
+
+void KMixWindow::increaseVolume(int percentage)
+{
+   Mixer *mixerMaster = Mixer::masterCard();
+   if (mixerMaster)
+   {
+      MixDevice *md = mixerMaster->masterDevice();
+      if (md)
+      {
+         mixerMaster->increaseVolume(md->num(), percentage);
       }
    }
 }
 
-void
-KMixWindow::increaseVolume()
+void KMixWindow::decreaseVolume(int percentage)
 {
-   Mixer* mixerMaster = Mixer::masterCard();
-   if ( mixerMaster != 0 ) {
-      MixDevice* md = mixerMaster->masterDevice();
-      if ( md != 0 ) {
-         mixerMaster->increaseVolume(md->num());
+   Mixer *mixerMaster = Mixer::masterCard();
+   if (mixerMaster)
+   {
+      MixDevice *md = mixerMaster->masterDevice();
+      if (md)
+      {
+         mixerMaster->decreaseVolume(md->num(), percentage);
       }
    }
 }
 
-void
-KMixWindow::decreaseVolume()
+int KMixWindow::volume()
 {
-   Mixer* mixerMaster = Mixer::masterCard();
-   if ( mixerMaster != 0 ) {
-      MixDevice* md = mixerMaster->masterDevice();
-      if ( md != 0 ) {
-         mixerMaster->decreaseVolume(md->num());
+   Mixer *mixerMaster = Mixer::masterCard();
+   if (mixerMaster)
+   {
+      return mixerMaster->masterVolume();
+   }
+   return -1;
+}
+
+void KMixWindow::setAbsoluteVolume(long absoluteVolume)
+{
+   Mixer *mixerMaster = Mixer::masterCard();
+   if (mixerMaster)
+   {
+      MixDevice *md = mixerMaster->masterDevice();
+      if (md)
+      {
+         return mixerMaster->setAbsoluteVolume(md->num(), absoluteVolume);
       }
    }
 }
 
+long KMixWindow::absoluteVolume()
+{
+   Mixer *mixerMaster = Mixer::masterCard();
+   if (mixerMaster)
+   {
+      MixDevice *md = mixerMaster->masterDevice();
+      if (md)
+      {
+         return mixerMaster->absoluteVolume(md->num());
+      }
+   }
+   return -1L;
+}
+
+long KMixWindow::absoluteVolumeMin()
+{
+   Mixer *mixerMaster = Mixer::masterCard();
+   if (mixerMaster)
+   {
+      MixDevice *md = mixerMaster->masterDevice();
+      if (md)
+      {
+         return mixerMaster->absoluteVolumeMin(md->num());
+      }
+   }
+   return -1L;
+}
+
+long KMixWindow::absoluteVolumeMax()
+{
+   Mixer *mixerMaster = Mixer::masterCard();
+   if (mixerMaster)
+   {
+      MixDevice *md = mixerMaster->masterDevice();
+      if (md)
+      {
+         return mixerMaster->absoluteVolumeMax(md->num());
+      }
+   }
+   return -1L;
+}
+
+void KMixWindow::setMute(bool on)
+{
+   Mixer *mixerMaster = Mixer::masterCard();
+   if (mixerMaster)
+   {
+      mixerMaster->setMasterMute(on);
+   }
+}
+
+void KMixWindow::toggleMute()
+{
+   Mixer *mixerMaster = Mixer::masterCard();
+   if (mixerMaster)
+   {
+      mixerMaster->toggleMasterMute();
+   }
+}
+
+bool KMixWindow::mute()
+{
+   Mixer *mixerMaster = Mixer::masterCard();
+   if (mixerMaster)
+   {
+      return mixerMaster->masterMute();
+   }
+   return true;
+}
+
+TQString KMixWindow::mixerName()
+{
+   Mixer *mixerMaster = Mixer::masterCard();
+   if (mixerMaster)
+   {
+      return mixerMaster->mixerName();
+   }
+   return TQString::null;
+}
+
+int KMixWindow::deviceIndex()
+{
+   Mixer *mixerMaster = Mixer::masterCard();
+   if (mixerMaster)
+   {
+      return mixerMaster->masterDeviceIndex();
+   }
+   return -1;
+}
+
+void KMixWindow::setBalance(int balance)
+{
+   Mixer *mixerMaster = Mixer::masterCard();
+   if (mixerMaster)
+   {
+      mixerMaster->setBalance(balance);
+   }
+}
+
 #include "kmix.moc"
 
