@@ -383,24 +383,23 @@ void Mixer::setMasterCard(const TQString& ref_id)
 
 Mixer* Mixer::masterCard()
 {
-  Mixer *mixer = 0;
   kdDebug(67100) << "Mixer::masterCard() searching for id=" << _masterCard << "\n";
-  for (mixer=Mixer::mixers().first(); mixer!=0; mixer=Mixer::mixers().next())
+  for (Mixer *mixer = Mixer::mixers().first(); mixer; mixer = Mixer::mixers().next())
   {
      if ( mixer->id() == _masterCard ) {
 #ifdef MIXER_MASTER_DEBUG
         kdDebug(67100) << "Mixer::masterCard() found id=" << mixer->id() << "\n";
 #endif
-        break;
+        return mixer;
      }
   }
 #ifdef MIXER_MASTER_DEBUG
-  if ( mixer == 0) kdDebug(67100) << "Mixer::masterCard() found no Mixer* mixer \n";
+  kdDebug(67100) << "Mixer::masterCard() found no Mixer* mixer \n";
 #endif
-  return mixer;
+  return NULL;
 }
 
-void Mixer::setMasterCardDevice(TQString& ref_id)
+void Mixer::setMasterCardDevice(const TQString &ref_id)
 {
   // The value is taken over without checking on existance. This allows the User to define
   // a MasterCard that is not always available (e.g. it is an USB hotplugging device).
@@ -515,6 +514,9 @@ void Mixer::setVolume( int deviceidx, int percentage )
   // @todo The next call doesn't handle negative volumes correctly.
   vol.setAllVolumes( (percentage*vol.maxVolume())/100 );
   _mixerBackend->writeVolumeToHW(deviceidx, vol);
+  // Make sure volume reading is synced
+  readSetFromHWforceUpdate();
+  TQTimer::singleShot(50, this, TQT_SLOT(readSetFromHW()));
 }
 
 /**
@@ -582,12 +584,15 @@ void Mixer::setAbsoluteVolume( int deviceidx, long absoluteVolume ) {
   Volume vol=mixdev->getVolume();
   vol.setAllVolumes( absoluteVolume );
   _mixerBackend->writeVolumeToHW(deviceidx, vol);
+  // Make sure volume reading is synced
+  readSetFromHWforceUpdate();
+  TQTimer::singleShot(50, this, TQT_SLOT(readSetFromHW()));
 }
 
 // @dcop , especially for use in KMilo
-long Mixer::absoluteVolume( int deviceidx )
+long Mixer::absoluteVolume(int deviceidx)
 {
-   MixDevice *mixdev= mixDeviceByType( deviceidx );
+   MixDevice *mixdev= mixDeviceByType(deviceidx);
    if (!mixdev) return 0;
 
    Volume vol=mixdev->getVolume();
@@ -596,9 +601,9 @@ long Mixer::absoluteVolume( int deviceidx )
 }
 
 // @dcop , especially for use in KMilo
-long Mixer::absoluteVolumeMax( int deviceidx )
+long Mixer::absoluteVolumeMax(int deviceidx)
 {
-   MixDevice *mixdev= mixDeviceByType( deviceidx );
+   MixDevice *mixdev= mixDeviceByType(deviceidx);
    if (!mixdev) return 0;
 
    Volume vol=mixdev->getVolume();
@@ -607,9 +612,9 @@ long Mixer::absoluteVolumeMax( int deviceidx )
 }
 
 // @dcop , especially for use in KMilo
-long Mixer::absoluteVolumeMin( int deviceidx )
+long Mixer::absoluteVolumeMin(int deviceidx)
 {
-   MixDevice *mixdev= mixDeviceByType( deviceidx );
+   MixDevice *mixdev= mixDeviceByType(deviceidx);
    if (!mixdev) return 0;
 
    Volume vol=mixdev->getVolume();
@@ -629,64 +634,77 @@ int Mixer::masterVolume()
 }
 
 // @dcop
-void Mixer::increaseVolume( int deviceidx )
+void Mixer::increaseVolume(int deviceidx, int percentage)
 {
-  MixDevice *mixdev= mixDeviceByType( deviceidx );
-  if (mixdev != 0)
+  MixDevice *mixdev= mixDeviceByType(deviceidx);
+  if (mixdev && percentage > 0)
   {
     Volume vol = mixdev->getVolume();
-    long inc = vol.maxVolume() / 20;
-    if (inc == 0)
+    long maxVol = vol.maxVolume();
+    if (maxVol > 0)
     {
-      inc = 1;
+      for (int i = 0; i < vol.count(); i++)
+      {
+        double perc = 100.0 * vol[i] / maxVol;
+        perc += percentage;
+        if (perc > 100.0)
+        {
+          perc = 100.0;
+        }
+        long newVal = (long)(perc * maxVol / 100.0);
+        mixdev->setVolume(i, newVal);
+      }
+      commitVolumeChange(mixdev);
     }
-    for (int i = 0; i < vol.count(); i++)
-    {
-      long newVal = (vol[i]) + inc;
-      mixdev->setVolume(i, newVal < vol.maxVolume() ? newVal : vol.maxVolume());
-    }
-    commitVolumeChange(mixdev);
   }
 }
 
 // @dcop
-void Mixer::decreaseVolume( int deviceidx )
+void Mixer::decreaseVolume(int deviceidx, int percentage)
 {
-  MixDevice *mixdev= mixDeviceByType( deviceidx );
-  if (mixdev != 0)
+  MixDevice *mixdev= mixDeviceByType(deviceidx);
+  if (mixdev && percentage > 0)
   {
     Volume vol = mixdev->getVolume();
-    long inc = vol.maxVolume() / 20;
-    if (inc == 0)
+    long maxVol = vol.maxVolume();
+    if (maxVol > 0)
     {
-      inc = 1;
+      for (int i = 0; i < vol.count(); i++)
+      {
+        double perc = 100.0 * vol[i] / maxVol;
+        perc -= percentage;
+        if (perc < 0.0)
+        {
+          perc = 0.0;
+        }
+        long newVal = (long)(perc * maxVol / 100.0);
+        mixdev->setVolume(i, newVal);
+      }
+      commitVolumeChange(mixdev);
     }
-    for (int i = 0; i < vol.count(); i++)
-    {
-      long newVal = (vol[i]) - inc;
-      mixdev->setVolume(i, newVal > 0 ? newVal : 0);
-    }
-    commitVolumeChange(mixdev);
   }
 }
 
 // @dcop
-void Mixer::setMute( int deviceidx, bool on )
+void Mixer::setMute(int deviceidx, bool on)
 {
-  MixDevice *mixdev= mixDeviceByType( deviceidx );
-  if (!mixdev) return;
+  MixDevice *mixdev= mixDeviceByType(deviceidx);
+  if (!mixdev)
+  {
+    return;
+  }
 
-  mixdev->setMuted( on );
-
-  _mixerBackend->writeVolumeToHW(deviceidx, mixdev->getVolume() );
+  mixdev->setMuted(on);
+  commitVolumeChange(mixdev);
 }
 
 // @dcop only
-void Mixer::setMasterMute( bool on )
+void Mixer::setMasterMute(bool on)
 {
-  MixDevice *master = masterDevice();
-  if (master != 0 ) {
-    setMute( master->num(), on );
+  MixDevice *md = masterDevice();
+  if (md)
+  {
+    setMute(md->num(), on);
   }
 }
 
@@ -744,10 +762,10 @@ bool Mixer::isRecordSource( int deviceidx )
   return mixdev->isRecSource();
 }
 
-/// @DCOP    WHAT DOES THIS METHOD?!?!?
+// @dcop
 bool Mixer::isAvailableDevice( int deviceidx )
 {
-  return mixDeviceByType( deviceidx );
+  return (mixDeviceByType(deviceidx) != NULL);
 }
 
 #include "mixer.moc"
